#ifndef ONI_SYMBOLS_H
#define ONI_SYMBOLS_H
#include "stdint.h"
#include <stdio.h>
#include "Oni.h"
#include "Oni_GL.h"
#include "Oni_Character.h"
#include "BFW_ScriptLang.h"

#include "patches/objt.h"

//Define variables of Oni: #define VARNAME (*((TYPE*)ADDRESS))

// AIs are deaf (ger: "taub")
#define ai2_deaf			(*((onibool*)0x005ec0c1))

// Option "debug" ... but what does it do?
#define AKgDebug_DebugMaps	(*((char*)0x002b2204))

// Option "debugfiles" ... but what does it do?
#define BFgDebugFileEnable	(*((bool*)0x0055c8d0))

// Parameter 1 to COrTextArea_Print. Context pointer?
#define COgConsoleLines		(*((uint32_t*)0x005cb468))

// Command line instance
#define COgCommandLine		(*((void**)0x00571b74))


// Default console text color
#define COgDefaultTextShade	(*((uint32_t*)0x00533f70))

// Default console text shadow color
#define COgDefaultTextShadow	(*((uint32_t*)0x00533f74))

// Console text fade timeout
#define COgFadeTimeValue	(*((uint32_t*)0x00533f68))

// Game termination flag (ONiRunGame loops on it).
#define ONgTerminateGame	(*((UUtBool*)0x00630ffd))

// Window manager is running (drawing and updating windows).
#define WMgActive			(*((UUtBool*)0x005e99a0))

// Set by WinMain, used by UUrPlatform_Initialize and ONrPlatform_Initialize,
// which copies it into ONgPlatformData.
#define ONgInstance			(*((HINSTANCE*)0x0061f9e4))

// Keeps initial gamma ramp value (before any change).
#define GLgInitialGammaRamp	(((M3tWin32GammaRamp*)0x0055fdfc))

// Nonzero when gamma ramp was successfully read into GLgInitialGammaRamp.
#define GLgGammaRampValid	(*((int*)0x005603fc))

// OpenGL: list of pointers to opengl32.dll functions (and a few functions from GL extensions)
#define gl_api				(*((gl_api_t**)0x00560604))

// OpenGL render engine descriptor (resolutions, flags, context methods)
#define gl				(*((gl_engine_t**)0x00560600))

// Initial display mode (saved by gl_library_is_loaded).
#define GLgInitialMode		(*((DEVMODE*)0x0055fd60))

// False to prevent Oni from changing display settings (however Bink
// player does not respect this setting).
#define M3gResolutionSwitch	(*((char*)0x00531634))

// Startup.txt file handle
#define ONgFileStartup		(*((FILE**)0x005711b8))

// Current GameState
#define ONgGameState		(*((GameState**)0x005ece7c))

// Current ONtPlatformData
#define ONgPlatformData		(*((ONtPlatformData*)0x00631008))

// Load non levelX_final-files yes/no
#define opt_ignore_private_data	(*((char*)0x006370f0))

// Play sound yes/no
#define opt_sound			(*((char*)0x006370fc))

// Option "findsounds" ... but what does it do?
#define SSgSearchOnDisk		(*((bool*)0x005eb758))

// OBJgTriggerVolume_Visible - bsl var to show trigger volumes
#define OBJgTriggerVolume_Visible	(*((bool*)0x005ec6c4))

//#define OBJgFlag_DrawFlags	(*((void*)0x005ec624))
//#define OBJgFlag_ViewPrefix	(*((void*)0x005ec650))
//#define OBJgFlag_DrawNameDistance	(*((void*)0x005ec634))

// Method signature for script (BSL) functions
typedef uint16_t	( ONICALL *sl_func)(sl_callinfo* callinfo, uint32_t numargs, sl_arg args[], int* dontuse1, int* dontuse2, sl_arg* ret);

// Used by function definition below: OBJrObjectType_EnumerateObjects
typedef char		( ONICALL *OBJtEnumCallback_Object)(void *inObject, int inUserData);



#define DefFunc( type, name, callingconvention, args, address) typedef type ( callingconvention * _##name ) args; static _##name name = (_##name)address

// Orig timing functions, replaced by DDrMachineTime_* if option "usegettickcount" is enabled
DefFunc(int64_t, UUrMachineTime_High, ONICALL, (), 0x04026480);
DefFunc(double, UUrMachineTime_High_Frequency, ONICALL, (), 0x040264b0);
DefFunc(int64_t, UUrMachineTime_Sixtieths, ONICALL, (), 0x040263e0);

// Code to initialize platform stuff (like disabling system key combination Alt+Tab)
DefFunc(void, UUrPlatform_Initialize, ONICALL, (), 0x04026010);
DefFunc(void, UUrPlatform_Terminate, ONICALL, (), 0x04026310);

// Oni's file access methods
DefFunc(FILE*, oni_fopen, __cdecl, (const char*, const char*), 0x0051ea9f);
DefFunc(int, oni_fprintf, __cdecl, (FILE*, const char*, ...), 0x0051ebbf);
DefFunc(int, oni_fflush, __cdecl, (FILE*), 0x0051eab2);

// Oni's main method?
DefFunc(void, ONiMain, __cdecl, (int ArgCount, char *ArgList[]), 0x004d3280);

// Initialization code, used to hook in windowed modes
DefFunc(short, ONrPlatform_Initialize, ONICALL, (ONtPlatformData *PlatformData), 0x0050f670);
DefFunc(LRESULT, ONrPlatform_WindowProc, CALLBACK, (HWND Window, UINT Message, WPARAM WParam, LPARAM LParam), 0x0050f7a0);

// Retrieves a list of resolutions, replaced by "daodan_enumerate_valid_display_modes"
DefFunc(unsigned int, gl_enumerate_valid_display_modes, ONICALL, (M3tDisplayMode modes[16]), 0x004083a0);
DefFunc(int, gl_platform_set_pixel_format, ONICALL, (HDC hdc), 0x00407b50);

// Original graphics initialization/cleanup functions patched to use
// DD_GLrPlatform_Initialize/DD_GLrPlatform_Dispoose instead if daodangl is used.
DefFunc(UUtBool, gl_platform_initialize, ONICALL, (void), 0x00407da0);
DefFunc(void, gl_platform_dispose, ONICALL, (void), 0x408210);

// Retrieve gamma value from settings
DefFunc(float, ONrPersist_GetGamma, ONICALL, (), 0x0050f450);

// Retrieve if the game was completed
DefFunc(uint8_t, ONrPersist_GetWonGame, ONICALL, (), 0x0050f660);

// Print a message to the startup.txt log
DefFunc(void, UUrStartupMessage, __cdecl, (const char* fmt, ...), 0x00424860);

// Probably to spawn a new character
DefFunc(int16_t, ONrGameState_NewCharacter, ONICALL, (void* CHAR, void* AISA, void* flag, uint32_t* list_location), 0x004dac50);

// Remove character
DefFunc(void, ONrGameState_DeleteCharacter, ONICALL, (Character *inCharacter), 0x004DC480);

DefFunc(void, ONrGameState_Timer_Start, ONICALL, (char* function, int time), 0x004FD370);

// Get active (player?) character
DefFunc(ActiveCharacter*, ONrGetActiveCharacter, ONICALL, (void* CharacterPtr), 0x004f1180);

DefFunc(uint32_t, ONrCharacter_GetHealthShade, ONICALL, (uint32_t health, uint32_t maxhealth), 0x004EF450);
DefFunc(void, ONrCharacter_NewAnimationHook, ONICALL, (Character *ioCharacter, ActiveCharacter *ioActiveCharacter), 0x004E97A0);
DefFunc(void, ONrCharacter_SetAnimationExternal, ONICALL, (Character *ioCharacter, short state, void* animation, int interpolation), 0x004EB340);
DefFunc(void, ONrCharacter_SetHitPoints, ONICALL, (Character *ioCharacter, uint32_t inHitPoints), 0x004EB220);
DefFunc(void, ONrCharacter_SetCharacterClass, ONICALL, (Character* Char, ONCC* Class), 0x004D7C30);
DefFunc(void, ONrCorpse_Create, ONICALL, (Character* Character), 0x004EF340);


// Print message to console
DefFunc(void, COrTextArea_Print, ONICALL, (uint32_t area, uint32_t priority, uint32_t textshade, uint32_t textshadowshade, const char* text, uint32_t unk_alwaws_0, uint32_t fadetime), 0x00431340);
DefFunc(int16_t, COrTextArea_Resize, ONICALL, (void* inTextArea, UUtRect* inBounds, int16_t inNumTextEntries), 0x00431460);

// Original cheat function
DefFunc(uint8_t, ONrCheater, ONICALL, (uint32_t cheat), 0x004f5c30);

DefFunc(int16_t, ONrMechanics_Register, ONICALL, (uint32_t inObjectType, uint32_t inObjectTypeIndex, char* inGroupName,
	uint32_t inSizeInMemory, OBJtMethods* inObjectMethods, uint32_t inFlags, void* inMechanicsMethods), 0x00507450);
DefFunc(int16_t, OBJrTriggerVolume_IntersectsCharacter, ONICALL, (const OBJtObject* inObject, uint32_t inMask, const void* inCharacter), 0x004cc680);


// ?
DefFunc(int, AUrMessageBox, __cdecl, (int Buttons, char *Message, ...), 0x004378c0);
DefFunc(char*, SSrMessage_Find, ONICALL, (char* message_key), 0x0047F550);
DefFunc(void, ONiGameState_FindAutoPromptMessage, ONICALL, (char* Note, void* ptr), 0x004FDBE0);

// Register a BSL function with a return type
DefFunc(uint16_t, SLrScript_Command_Register_ReturnType, ONICALL, (char* name, char* desc, char* argfmt, sl_type type, sl_func callback), 0x00477b20);

// Register a BSL function without a return value
DefFunc(uint16_t, SLrScript_Command_Register_Void, ONICALL, (char* name, char* desc, char* argfmt, sl_func callback), 0x00477b40);

// Register a global BSL variable
DefFunc(uint16_t, SLrGlobalVariable_Register_Bool, ONICALL, (char* name, char* desc, UUtBool* data), 0x00477f50);
DefFunc(uint16_t, SLrGlobalVariable_Register_Int32, ONICALL, (char* name, char* desc, int32_t* data), 0x00477e30);
DefFunc(uint16_t, SLrGlobalVariable_Register_Float, ONICALL, (char* name, char* desc, float* data), 0x00477ec0);
DefFunc(uint16_t, SLrGlobalVariable_Register_String, ONICALL, (char* name, char* desc, char* data), 0x00477fe0);

// Print message to console ?
DefFunc(int, COrMessage_Print, ONICALL, (char* Message, char* Key, void* noidea), 0x004304B0);
DefFunc(UUtBool, COrCommand_Execute, ONICALL, (char* command), 0x004317D0);

DefFunc(void, COrConsole_StatusLine_Display, ONICALL, (), 0x00431E70);

// Get pointer to specified data instance
DefFunc(int16_t, TMrInstance_GetDataPtr, ONICALL, (int tag, char* name, void* pointer), 0x004232E0);

// Get name of data instance pointed to
DefFunc(char*, TMrInstance_GetInstanceName, ONICALL, (void* InstancePointer), 0x00423D90);

DefFunc(short, TMrInstance_GetDataPtr_ByNumber, ONICALL, (int tag, int number, void** out), 0x00423680);
DefFunc(uint32_t, TMrInstance_GetTagCount, ONICALL, (int tag), 0x004236F0);

// Draw text on the screen
DefFunc(int16_t, TSrContext_DrawText, ONICALL, (uint32_t TSrContext, char* Text, char alpha, uint32_t usuallyzero, void* pRect), 0x0042DF00);
DefFunc(int16_t, TSrContext_New, ONICALL, (void* FontInstance, int size, int hthsik1,int hthsik2,int hthsik3, void* TSrContext), 0x0042EA30);
DefFunc(int16_t, TSrContext_SetFontSize, ONICALL, (void* ioTextContext, uint16_t inSize), 0x0042ED50);
DefFunc(int16_t, TSrContext_SetShade, ONICALL, (void* ioTextContext, uint32_t inShade), 0x0042EE50);

DefFunc(uint16_t, TRrAnimation_GetDuration, ONICALL, (void* Animation), 0x00428740);
DefFunc(uint16_t, TRrAnimation_GetTo, ONICALL, (void* Animation), 0x00428730);
DefFunc(uint16_t, TRrAnimation_GetFrom, ONICALL, (void* Animation), 0x00428720);

DefFunc(uint16_t, iSetCharacterClass, ONICALL, (sl_callinfo* callinfo, uint32_t numargs, sl_arg args[], int* dontuse1, int* dontuse2, sl_arg* ret), 0x004D99D0);
DefFunc(uint16_t, AI2iScript_Spawn, ONICALL, (sl_callinfo* callinfo, uint32_t numargs, sl_arg args[], int* dontuse1, int* dontuse2, sl_arg* ret), 0x004B4780);
DefFunc(void, AI2rSmite, ONICALL, (Character *inCharacter, UUtBool inSpareCharacter), 0x0048BDA0);
//NOT USED: DefFunc(void*, OBJrObjectType_GetObject_ByNumber, ONICALL, (int inObjectType, int inIndex), );
//NOT USED: DefFunc(int, OBJiObjectGroup_GetNumObjects, ONICALL, (void *inObjectGroup), );

DefFunc(int, OBJrObjectType_EnumerateObjects, ONICALL, (int inObjectType, OBJtEnumCallback_Object inEnumCallback, int inUserData), 0x004D0080);
//DefFunc(void, OBJrTriggerVolume_IntersectsCharacter, ONICALL, (), 0x004cc680);

DefFunc(void, OBJrDoor_Open, ONICALL, (DoorObject *inObject, Character *inCharacter), 0x004C26C0);
DefFunc(void, OBJrDoor_ForceOpen, ONICALL, (short id), 0x004C1EE0);
DefFunc(short, OBJrConsole_OnActivate, ONICALL, (void *inObject, Character *inCharacter), 0x004C0880);
DefFunc(void*, OBJrConsole_GetByID, ONICALL, (short ID), 0x004C0950);


DefFunc(void, ONiDrawWeaponSight, ONICALL, (Character* Char), 0x004E1900);
DefFunc(void, AI2rDisplayDebuggingInfo, ONICALL, (Character* Char), 0x0048C5F0);
DefFunc(uint32_t, M3rTextureMap_New, ONICALL, (short width, short height, int type, int allocated, int flags, char* name, void** output), 0x0041EB00);
DefFunc(void, M3rDraw_BigBitmap, ONICALL, (M3tTextureMap_Big* inBigBitmap, const M3tPointScreen *inDestPoint, UUtUns16 inWidth, UUtUns16 inHeight, UUtUns32 inShade, UUtUns16 inAlpha), 0x0041f6e0);
DefFunc(void, M3rDraw_Bitmap, ONICALL, (void* inBitmap, const M3tPointScreen* inDestPoint, UUtUns16 inWidth, UUtUns16 inHeight, UUtUns32 inShade, UUtUns16 inAlpha), 0x0041f4a0);

DefFunc(void, M3rGeom_Line_Light, ONICALL, (const M3tPoint3D *point1, const M3tPoint3D *point2, uint32_t shade), 0x0041f440);
/*
DefFunc(void, M3rGeom_Line_Light, ONICALL, (), 0x0041f440);
DefFunc(void, M3rMatrixStack_Push, ONICALL, (), 0x0041baa0);
DefFunc(void, M3rMatrixStack_Get, ONICALL, (), 0x0041bb20);
DefFunc(void, M3rMatrixStack_Pop, ONICALL, (), 0x0041bb30);
DefFunc(void, M3rMatrixStack_Identity, ONICALL, (), 0x0041bba0);
DefFunc(void, M3rMatrixStack_Rotate, ONICALL, (), 0x0041bbe0);
DefFunc(void, M3rMatrixStack_UniformScale, ONICALL, (), 0x0041bc20);
DefFunc(void, M3rMatrixStack_Translate, ONICALL, (), 0x0041bc40);
DefFunc(void, M3rMatrixStack_Multiply, ONICALL, (), 0x0041bc70);
DefFunc(void, M3rGeom_State_Commit, ONICALL, (), 0x0041e870);

DefFunc(void, MUrPoint_Distance, ONICALL, (), 0x00433270);
*/

// Make a dialog element (in)visible and change its position
DefFunc(void, WMrWindow_SetVisible, ONICALL, (WMtWindow* window, int visibility), 0x00475a10);
DefFunc(void, WMrWindow_SetLocation, ONICALL, (WMtWindow* window, int x, int y), 0x004756d0);

// Plays movie using Bink blitting method (ie., non-OpenGL blitting).
DefFunc(void, ONrMovie_Play, ONICALL, (const char* movie, int unknown), 0x004d5580);

// Sets minimum and maximum values for a slider control.
DefFunc(void, WMrSlider_SetRange, ONICALL, (WMtWindow* window, int min_value, int max_value), 0x00472650);

// Enables or disables window.
DefFunc(void, WMrWindow_SetEnabled, ONICALL, (WMtWindow* window, UUtBool enabled), 0x00475580);

// Sets gamma to specified factor (0.0 .. 1.0).
DefFunc(void, M3rSetGamma, ONICALL, (float factor), 0x00407a60);

// Resets gl_api pointers, unloads OPENGL32.DLL.
DefFunc(void, gl_unload_library, ONICALL, (void), 0x0040ac40);

// Notifies platform-specific local input implementation about internal (ie.,
// real) input mode change.
DefFunc(void, LIrPlatform_Mode_Set, ONICALL, (unsigned int active_mode), 0x00402cf0);

// Terminates platform-specific local-input.
DefFunc(void, LIrPlatform_Terminate, ONICALL, (void), 0x00403620);

#undef DefFunc

#endif
